;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;  Character Averaging Routines
     CHROMA & CHR$128 Version
;     Gregory C. Harder 2020
;================================

ORG 4084

@Keyboard=02BB			;Keyboard scanning subroutine
@Laddr=09D8			;Get line address subroutine
@Seed=4032			;16bit parameter passing via RAND
@DFile=400C			;Start of DFile
@Frames=4034			;Pseudo-random number
@Ireg=403C			;16444, Ireg value
@Bordr=403D			;16445, Border pattern
@Dlay=403E			;16446, Delay value
@Pattern=403F			;16447, Pattern selected
@Cells=4040			;16448, Cell neighborhood data, 9 bytes
@Cycle=4049			;16457, Cycle style
;================================
; Jump Table
;================================
	JR SetUp		;16516, Font
	JR Fill			;16518, Fill screen with random chrs
	JR Border		;16520, Draw border
	JP Invert		;16522, Ink/Paper swap
	JP CellCopy		;16525, Copy patterns to font area
	JP Analyze		;16528, Analyze the neighborhood
	JP RotIn		;16531, Cycle in loop
	JP RotOut		;16534, Cycle out loop
	JP Pulse		;16537, Pulse cycle loop
	JP OneStep		;16540, Single step cycle
;================================
; Install default pixel patterns
;================================
@Pixels				;16543
	LD HL,3208		;Default patterns
	LD DE,3008		;Destination
	LD BC,0078		;Byte count
	LDIR			;Copy default patterns
	RET
;================================
; Set up font and UDGs
;================================
@SetUp
	CALL Loader		;Set up color display
	LD HL,0002		;Locate line 2
	CALL LAddr		;Get line address
	LD BC,000D		;Skip header
	ADD HL,BC
	LD DE,3000		;UDG patterns
	LD BC,0400		;Byte count
	LDIR			;Copy font patterns
	CALL Pixels		;Install default pixel patterns
	LD A,(Ireg)		;Get Ireg value
	LD I,A			;Set Ireg
	JP Release
;================================
; Fill screen with random chrs
;================================
@Fill
	LD A,(Dlay)		;Get delay value
	LD (Delay+2),A		;Set delay value
	LD HL,(DFile)		;Get DFile
	LD BC,0300		;Byte count, 24*32=768
@FillUp
	CALL Rnd8bit		;Get random number, 0 to 255
	AND 0F			;AND 0000 1111, range=0 to 15
	JR Z FillUp		;If 0 try again
@FillScr
;	A=1 to 15
	INC HL			;Next DFile addr
	BIT 6,(HL)		;N/L?
	JR NZ +4		;Yes, skip it
	LD (HL),A		;Set chr
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ FillUp		;Loop until BC=0
;==============================
; Print border
;==============================
@Border
	LD A,(Bordr)		;Get border color
	LD HL,(DFile)		;Get DFile
	LD BC,1700		;Row counter, 24
@Bord1
	INC HL 			;Next DFile addr
	LD (HL),A		;Set left border
	LD DE,001F		;Offset to right border
	ADD HL,DE 		;Right border addr
	LD (HL),A		;Set right border
	INC HL 			;Skip N/L
	DJNZ Bord1		;Loop until B=0
	CALL TopBot		;Print bottom row
	LD HL,(DFile)		;Get DFile
@TopBot
	INC HL			;Next DFile addr
	LD B,20			;Column counter, 32
@Bord2
	LD (HL),A		;Set bottom/top border
	INC HL			;Next DFile addr
	DJNZ Bord2		;Loop until B=0
	RET
;================================
; Invert patterns
;
; Swap B&W
;================================
@Invert
	LD HL,3008		;Patterns
	LD B,78			;Byte count
	LD A,(HL)		;Get pattern
	CPL			;Compliment the patterns
	LD (HL),A		;Set pattern
	INC HL			;Next pattern addr
	DJNZ Invert+5		;Loop until B=0
	JP Release		;Return via Release
;================================
; Copy selected patterns
;================================
@CellCopy
	LD HL,0003		;Line number
	CALL Laddr		;Locate line addr
	LD BC,000D		;Header bytes
	ADD HL,BC		;Skip header
	EX DE,HL		;DE=Start of data
	LD HL,(Seed)		;Get offset
	ADD HL,DE
	LD C,78			;Byte count, 120
	SBC HL,BC		;HL=Pattern addr
	LD DE,3008		;Cell patterns
	LDIR			;Copy cell patterns
	RET
;================================
; Analyze cell condition
;================================
@Analyze
	LD HL,(DFile)		;Get DFile
	LD DE,0022		;Offset to NW corner of grid
	ADD HL,DE		;NW corner of grid-1
	LD C,16			;Row count
@NextY
	LD B,1E			;Byte count
@NextX
	INC HL			;Next DFile addr
;==============================
; Copy cell data to storage area
;==============================
	PUSH HL			;PUSH cell home addr
	PUSH BC			;PUSH counters
	LD A,03			;Byte count
	LD BC,0022		;Offset to top row
	AND A			;Clear CY
	SBC HL,BC		;Start of top row
	LD C,A			;Byte counter
	LD DE,Cells		;Storage area
	LDIR			;Copy top row
	LD C,1E			;Offset to middle row
	ADD HL,BC		;Start of middle row
	LD C,A			;Byte count
	LDIR			;Copy middle row
	LD C,1E			;Offset to bottom row
	ADD HL,BC		;Start of Bottom row
	LD C,A			;Byte count
	LDIR			;Copy bottom row
;==============================
; Total up the neighborhood
;==============================
	LD B,09			;Byte counter, 9 cells
	LD DE,Cells		;Start of stored data
	LD HL,0000		;Accumulator
@AddUp
	PUSH BC			;PUSH counter
	LD B,00
	LD A,(DE)		;Get value
	LD C,A			;Copy to C
	ADD HL,BC		;Add to accumulator
	INC DE			;Next data point
	POP BC			;POP counter
	DJNZ AddUp		;Loop until B=0
;==============================
; Divide the total by 9 to get 
; average value, at this point
; HL=Cells total
;==============================
@Divide
	XOR A			;A=0
	LD DE,0009		;Divisor=9
@DLup
	SBC HL,DE		;Subtract divisor from total
	INC A			;Increase counter
	JR NC DLup		;Loop until negative value reached
	DEC A			;Decrease counter, A=INT(Total/Divisor)
	POP BC			;POP Counters
	POP HL			;POP home cell addr
	LD (HL),A		;Set averaged chr
	DJNZ NextX		;Loop until B=0
	INC HL			;Start of next row
	INC HL
	INC HL
	DEC C			;Decrease row counter
	JR NZ NextY		;Loop until C=0
	RET
;================================
; Rotate In
;================================
@RotIn
	CALL Release		;Wait for key release
	CALL CycleIn		;Rotate In
	CALL TestKey		;Keypress?
	JR Z RotIn		;Loop until key pressed
@GOTO20
	CALL Release		;Wait for key release
	LD C,14			;GOTO 20 on return
	RET
;================================
; Cycle In
;================================
@CycleIn
	LD A,(Cycle)		;Get cycle style
	AND A			;0?
	JR Z PatIn		;Yes, static color
	CP 01			;1?
	JR NZ +4		;No, move on
	LD A,C9			;Yes, static patterns
	JR +1
	XOR A			;Rotate color and pattern
	LD (PatIn-1),A		;Set as NOP or RET instruction
;Rotate colors
	LD HL,C008		;Color attributes table+8
	LD B,70			;Byte count, B=14*8=112
@InLup
	LD A,(HL)		;Get attribute
	INC A			;Cycle to next attribute
	CP 08			;D.Grey?
	JR NZ +3		;No, move on
	INC A			;8 (D.Grey) goes to 9 (Blue)
	JR +6			;Move on
	CP 10			;16?
	JR NZ +2		;No, move on
	LD A,01			;16 goes to 1 (D.Blue)
	LD (HL),A		;Set attribute
	INC HL			;Next attribute addr
	DJNZ InLup		;Loop until B=0
	00			;NOP or RET instruction
;Rotate patterns
@PatIn
	LD HL,3008		;Chr 1 pattern
	LD DE,Cells		;Store it here
	PUSH DE			;PUSH stored chr addr
	LD BC,0008		;Byte count
	LDIR			;Copy chr 1 to storage
;	HL=3010			;Chr 2 addr
	LD DE,3008		;Chr 1 addr
	LD C,70			;Byte count, B=14*8=120
	LDIR			;Copy patterns
	POP HL			;POP stored chr addr
	LD C,08			;Byte count
	LDIR			;Copy chr 1 to chr 15
	RET
;================================
; Rotate Out
;================================
@RotOut
	CALL Release		;Wait for key release
	CALL CycleOut		;Rotate out
	CALL TestKey		;Keypress?
	JR Z RotOut		;Loop until key pressed
	JR GOTO20		;Return via GOTO20
;================================
; Cycle Out
;================================
@CycleOut
	LD A,(Cycle)		;Get cycle style
	AND A			;0?
	JR Z PatOut		;Yes, static color
	CP 01			;1?
	JR NZ +4		;No, move on
	LD A,C9			;Yes, statis patterns
	JR +1
	XOR A			;Rotate color and pattern
	LD (PatOut-1),A		;Set as NOP or RET instruction
;Rotate colors
	LD HL,C008		;Color attributes table+8
	LD B,70			;Byte count, B=14*8=112
@OutLup
	LD A,(HL)		;Get attribute
	DEC A			;Cycle to next attribute
	CP 08			;D.Grey?
	JR NZ +3		;No, move on
	DEC A			;8 (D.Grey) goes to 7 (Grey)
	JR +5			;Move on
	AND A			;0?
	JR NZ +2		;No, move on
	LD A,0F			;0 goes to 15 (White) 
	LD (HL),A		;Set attribute
	INC HL			;Next attribute addr
	DJNZ OutLup		;Loop until B=0
	00			;NOP or RET instruction
;Rotate patterns
@PatOut
	LD HL,307F		;Last byte of chr 15
	LD DE,Cells+7		;Store it here
	LD BC,0008		;Byte count
	LDDR			;Copy chr 15 to storage
;	HL=3078			;Chr 15 addr
	LD DE,307F		;Last pattern addr
	LD C,70			;Byte count, B=14*8=120
	LDDR			;Copy patterns
	LD HL,Cells		;Stored cell 15 pattern
	LD DE,3008		;Chr 1 addr
	LD C,08			;Byte count
	LDIR			;Copy chr 15 to chr 1
	RET
;================================
; Pulse Cycle
;================================
@Pulse
	LD B,0F			;Counter
@PulseIn
	PUSH BC			;PUSH counter
	CALL CycleIn		;Rotate In
	CALL Delay		;Delay loop
	POP BC			;POP counter
	DJNZ PulseIn		;Loop until B=0
	LD B,0F			;Counter
@PulseOut
	PUSH BC			;PUSH counter
	CALL CycleOut		;Rotate Out
	CALL TestKey		;Check for key press
	POP BC			;POP counter
	JP NZ GOTO20		;Return via GOTO20
	DJNZ PulseOut		;Loop until B=0
	JR Pulse		;Loop until keypress
;================================
; Single step cycle
;================================
@OneStep
	CALL Release		;Wait for key release
	CALL CycleIn		;Rotate In
	JP GOTO20		;Return via GOTO20
;================================
; Delay Loop
;================================
@Delay
	LD BC,0001
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ Delay+3		;Loop until BC=0
	RET
;================================
; Test for key press
;================================
@TestKey
	CALL Delay
	Call Keyboard		;Call ROM scan keybd.
	INC L			;Check for no key press, L=FF
	RET
;================================
; Test for no key press
;================================
@Release
	CALL Keyboard		;Scan keyboard routine.
	INC L			;
	JR NZ Release		;L<>0? Scan again
	LD BC,0000
	RET
;================================
; 8-Bit Random Number Generator
;
; For best results this should not
; be RUN in FAST made.
;
;  BC not used
;
; On exit...
;
;  HL Unchanged
;  DE Unchange
;  A Random number, range 0 to 255
;================================
@Rnd8Bit
	PUSH HL			;PUSH HL
	PUSH DE			;PUSH DE
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+3),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	POP DE			;POP DE
	POP HL			;POP HL
	RET
;================================
;     Color Loader Routines
;        by Paul Farrow
;================================
@Loader
	LD BC,7FEF 		;Chroma test
	IN A,(C) 		;Hello Chroma!
	BIT 5,A 		;Chroma answered?
	RET NZ 			;No, return
	LD HL,C000 		;Start of color attributes table
	LD D,H 			;Set all values to 0
	LD E,01 
	LD BC,03FF 
	LD (HL),L 
	LDIR 
	LD HL,Colors 		;User defined color table
@Loader1
	LD A,(HL) 		;Get chr code
	BIT 7,A 		;End of table?
	JR NZ Mode		;Yes, set mode
	INC HL 			;Get chr count
	LD D,18 		;Locate chr position in attribute table
	ADD A,A 
	RL D 
	ADD A,A 
	RL D 
	ADD A,A 
	RL D 
	LD E,A 
	LD A,(HL) 		;Set chr count
	INC HL 
@Loader2
	PUSH HL 		;PUSH table pointer
	LD C,08 		;Byte count
	LDIR 			;Copy data to attribute table
	POP HL 			;POP table pointer
	DEC A 			;Decrease chr counter
	JR NZ Loader2		;Loop until A=0
	LD C,08 		;Offset
	ADD HL,BC 		;Next position in table
	JR Loader1		;Loop until end of color table reached
@Mode
	SUB 60			;0110 0000 Character code mode+black border 
	LD BC,7FEF 
	OUT (C),A 
	IN A,FE
	RET
;================================
; Color table, C008 to C07F
;
; Format...
;
;  Chr Code/Count/Attribute (8 Bytes)
;================================
@Colors

00 80 0C 0C 0C 0C 0C 0C 0C 0C 	;Set all chrs to default colors
01 01 09 09 09 09 09 09 09 09 	; gA  Blue
02 01 0A 0A 0A 0A 0A 0A 0A 0A	; gD  Red
03 01 0B 0B 0B 0B 0B 0B 0B 0B	; gS  Magenta
04 01 0C 0C 0C 0C 0C 0C 0C 0C	; "=  Green
05 01 0D 0D 0D 0D 0D 0D 0D 0D	; =Cyan
06 01 0E 0E 0E 0E 0E 0E 0E 0E	; $=Yellow
07 01 0F 0F 0F 0F 0F 0F 0F 0F	; :=White
08 01 01 01 01 01 01 01 01 01	; g1  D.Blue
09 01 02 02 02 02 02 02 02 02	; g2  D.Red
0A 01 03 03 03 03 03 03 03 03	; g7  D.Magenta
0B 01 04 04 04 04 04 04 04 04	; g4  D.Green
0C 01 05 05 05 05 05 05 05 05	; g5  D.Cyan
0D 01 06 06 06 06 06 06 06 06	; gT  D.Yellow
0E 01 07 07 07 07 07 07 07 07	; gE  D.White (Grey)
12 01 0A 0A 0A 0A 0A 0A 0A 0A	; >   Red
17 01 0B 0B 0B 0B 0B 0B 0B 0B	; *   Magenta
80				; End of table marker
