;=====================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;=====================================
;    Spreading Fire Cellular Automata
;         Gregory C. Harder 2020
;=====================================

ORG 4084

@DFile=400C			;DFile sysvar
@Frames=4034			;Seed
@Fire=+16444			;Fire chr
@Ember=+16445			;Ember chr
@Ash=+16446			;Water chr
@H2O=+16447			;Water chr
@BKG=+16448			;Background chr
@Prob=+16449			;H2O density/Burn Probability
@Speed=+16450			;Burn speed
;================================
	JR Border		;16516, Draw border
	JR CLS			;16518,	CLS with BKG chr
	JR Water		;16520, Random screen of water
	JP AshCount		;16522, Count ash chrs
	JP Analyze		;16525, Cell analysis
	JR VidTest		;16528, Test Reg I for video type
;================================
; "POKE" chrs			;16530, Poke chr definitions
;================================
	LD HL,Fire		;Fire chr
	LD A,(HL)		;Get fire chr
	LD (Fire1+1),A		;Set fire chr
	LD (Fire2+1),A		;Set fire chr
	LD (Fire3+1),A		;Set fire chr
	INC HL			;Ember chr
	LD A,(HL)		;Get ember chr
	LD (Ember1+1),A		;Set ember chr
	LD (Ember2+1),A		;Set ember chr
	LD (Ember3+1),A		;Set ember chr
	INC HL			;Ash chr
	LD A,(HL)		;Get ash chr
	LD (Ash1+1),A		;Set ash chr
	LD (Ash2+1),A		;Set ash chr
	LD (Ash3+1),A		;Set ash chr
	INC HL			;Water chr
	LD A,(HL)		;Get water chr
	LD (Water1+1),A		;Set water chr
	LD (Water2+1),A		;Set water chr
	INC HL			;BKG chr
	LD A,(HL)		;Get background chr
	LD (BKG1+1),A		;Set background chr
	LD BC,0000		
	RET			;Ret 0 to BASIC
;==============================
; Video test
;==============================
@VidTest
	LD BC,0000		;0
	LD A,I			;Get REG I
	CP 1E			;Standard edition?
	RET Z			;Yes, return 0
	INC C			;1
	CP 21			;UDG?
	RET Z			;Yes, return 1
	INC C			;Must be Chroma, return 2
	RET
;================================
; Clear screen with BKG chr
;================================
@CLS
	LD HL,(DFile)		;Get DFile
	LD BC,0318		;Byte count
@CSLup
	INC HL			;Next DFile addr
	BIT 6,(HL)		;N/L?
	JR NZ +2		;Yes, skip it
@BKG1
	LD (HL),00		;Fill with BKG chr
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ CSLup		;Loop until BC=0
	RET
;==============================
; Print border
;==============================
@Border
	LD A,80			;Border character
	LD HL,(DFile)		;Get DFile
	LD BC,1700		;24 Rows
@Line
	INC HL 
	LD (HL),A		;Border Chr, left edge
	LD DE,001F
	ADD HL,DE 
	LD (HL),A		;Border Chr, right edge
	INC HL 			;Skip over N/L
	DJNZ LINE
	CALL Row
	LD HL,(DFile)
@Row
	INC HL
	LD B,20
@RowLup
	LD (HL),A		;Border Chr
	INC HL
	DJNZ RowLup
	RET
;================================
; Generate a screen of random water
;================================
@Water
	LD A,(Prob)		;Get water density
	LD (Prob1+1),A		;Set water density
	LD HL,(DFile)		;Get DFile
	LD BC,0318		;Byte count
@WaterLup
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ Water1+2		;Yes, skip it
	PUSH HL			;PUSH DFile
	CALL RND8Bit		;Get random number, 0 to 255
	POP HL			;POP DFile
@Prob1
	CP 00			;Test against probability
	JR NC +2		;No CY no water
@Water1
	LD (HL),00		;Print water chr
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ WaterLup		;Loop until BC=0
	RET
;==============================
; Count burnt
;==============================
@AshCount
	LD HL,(DFile)		;Get DFile
	LD BC,0021		;Skip title bar
	ADD HL,BC
	LD DE,02D6		;Byte counter
	LD BC,0000		;Ash accumulator
@AshLup
	INC HL			;Next DFile
	LD A,(HL)		;Get status
@Ash1
	CP 00			;Ash chr?
	JR NZ +1		;No, move on
	INC BC			;Increase ash count
	DEC DE 			;Decrease counter
	LD A,D 
	OR E 
	JR NZ AshLup		;Loop until DE=0
	RET			;Return ash count to BASIC
;================================
; Analyze cell condition
;================================
@Analyze
	LD A,(Speed)		;Get wind speed
	LD (Wind+2),A		;Set wind speed
	LD A,(Prob)		;Get burn probability
	LD (Prob2+1),A		;Set burn probability
	LD HL,(DFile)		;Get DFile
	LD BC,0021		;Skip title bar
	ADD HL,BC
	INC HL			;NW corner of grid
	PUSH HL
	LD DE,02D6		;Cell counter
@Spread
	PUSH DE			;PUSH cell counter
	INC HL			;Next Cell
	LD (Ignite+1),HL	;Save cell addr
	LD A,(HL)		;Get cell status
	CP 76			;N/L?
	JR Z NextCell		;Yes, skip it
@Water2
	CP 00			;Water?
	JR Z NextCell		;Yes, skip it
@Ember1
	CP 00			;Ember?
	JR NZ +4		;No, move on
@Ash2
	LD (HL),00		;Ember->Ash
	JR NextCell
@Fire1
	CP 00			;Burning cell?
	JR NZ +4		;No, move on
@Ember2
	LD (HL),00		;Burning->Ember
	JR NextCell
@Ash3
	CP 00			;Ash?
	JR Z NextCell		;Yes, skip it
	BIT 7,A			;Border chr?
	JR NZ NextCell		;Yes, skip it
;==============================
; Analyze the surrounding cells
;==============================
	AND A			;NWcell
	SBC HL,BC
	DEC HL
	CALL Burn
	INC HL			;Ncell
	CALL Burn
	INC HL			;NEcell
	CALL Burn
	ADD HL,BC		;Ecell
	CALL Burn
	ADD HL,BC		;SEcell
	CALL Burn
	DEC HL			;Scell
	CALL Burn
	DEC HL			;SWcell
	CALL Burn
	AND A			;Ecell
	SBC HL,BC
	CALL Burn
@NextCell
	LD HL,(Ignite+1)	;Get current cell position
	POP DE			;POP cell counter
	DEC DE 			;Decrease cell counter
	LD A,D 
	OR E 
	JR NZ Spread		;Loop until DE=0
@NoFire				;Test for fire out
	POP HL			;POP DFile addr, NW corner of grid
	LD DE,02D6		;Byte counter
	LD BC,0001		;Set for return
@Burning
	INC HL			;Next DFile
	LD A,(HL)		;Get status
@Ember3
	CP 00			;Ember?
	RET Z			;Yes, return 1 if ember
	DEC DE 			;Decrease counter
	LD A,D 
	OR E 
	JR NZ Burning		;Loop until DE=0
	DEC C			;BC=0
	RET			;Return 0 if out
;==============================
; Start a fire based on probability
; 
; BC unchange
;==============================
@Burn
	LD A,(HL)
@Fire2
	CP 00			;Burning Neighbor
	RET NZ			;No, return
	CALL RND8Bit		;Get random number, 0 to 255
@Prob2
	CP 00			;Test against probability
	RET NC			;IF CY no fire
@Ignite
	LD HL,0000		;Cell address
@Fire3
	LD (HL),00		;Set fire
@Wind
	LD DE,0001		;Wind speed
	DEC DE 			;Decrease counter
	LD A,D 
	OR E
	JR NZ Wind+3		;Loop until DE=0
	POP AF			;POP return address
	JR NextCell		;Jump, we don't need to test anymore neighbors
;================================
; 8-Bit Random Number Generator
; Reg BC not used
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET