;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;     Forest Fire Simulation
;     Gregory C. Harder 2020
;================================

ORG 4084

@Laddr=09D8			;Find line address subroutine
@DFile=400C			;DFile sysvar
@Frames=4034			;Seed

@Speed=+16444			;Wind speed
@Burn=+16445			;New fire probability
@Seed=+16446			;New tree probability
@Density=+16447			;Tree/water density
@FVar=+16448			;New "F"ire probability
@PVar=+16449			;New "P"lant probability
@Fire=95			;Fire chr
@H2O=08				;Water chr
@Tree=15			;Tree chr

	JR RandomScr		;16516
	JR RandomH2O		;16518
	JR Analyze		;16520
;==============================
; Test for CHR$128 mode
;==============================
@CHR128				;16522
	LD BC,0000		;Set for return
	LD A,I			;Get REG I
	CP 21			;CHR$128?
	RET NZ			;No, return
	LD HL,2040		;Water chr
	LD B,08			;Byte count
@CPLH2O
	LD A,(HL)		;Complimet pattern for CHR$128
	CPL
	LD (HL),A
	INC HL
	DJNZ CPLH2O
	RET
;================================
; Generate a 32x24 random screen
;================================
@RandomScr
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD BC,0300		;Byte count
@FLup
	PUSH HL			;PUSH DFile address
	CALL Rnd8Bit		;Get random number
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;IF L>Density print tree/water
@Water
	LD A,"+"		;Tree/water chr.
	POP HL			;POP DFile address
	JR C +1			;If CY then skip print
	LD (HL),A		;Print tree/water
	INC HL			;Next DFile address
	BIT 6,(HL)		;N/L?
	JR Z +1			;No, move on
	INC HL			;Yes, Skip it
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ FLup		;Loop until BC=0
	JR Border		;Print Border
;==============================
; Random water
;==============================
@RandomH2O
	LD A,H2O			;Water chr
	LD (Water+1),A		;Set water chr
	CALL RandomScr		;Generate water screen
	LD A,"+"		;Restore tree chr
	LD (Water+1),A		;Set tree chr
	RET
;==============================
; Print border
;==============================
@Border
	LD A,80			;Border character
	LD HL,(DFile)
	LD B,17 		;24 Rows
@Line
	INC HL 
	LD (HL),A		;Border Chr
	LD DE,001F
	ADD HL,DE 
	LD (HL),A		;Border Chr
	INC HL 
	DJNZ LINE
	CALL Row
	LD HL,(DFile)
@Row
	INC HL
	LD B,20
@RowLup
	LD (HL),A		;Border Chr
	INC HL
	DJNZ RowLup
	LD C,00
	RET
;================================
; Analyze cell condition
;================================
@Analyze
	LD A,(Speed)		;Get wind speed
	LD (Wind+2),A		;Set wind speed
	LD A,(PVar)		;Get new tree probability
	LD (NewTree+1),A	;Set new tree probability
	LD A,(FVar)		;Get new fire probability
	LD (NewFire+1),A	;Set new fire probability
	LD HL,(DFile)		;Get DFile
	INC HL			;Start of DFile
	LD BC,0021		;Skip title bar
	ADD HL,BC		;DFile row 2
	LD DE,02D3		;Cell counter
@Spread
	PUSH DE			;PUSH cell counter
	INC HL			;Next Cell
	LD (CenterCell+1),HL	;Save cell addr
	LD A,(HL)		;Get cell status
	CP 76			;N/L?
	JR Z NextCell		;Yes, skip it
	CP H2O			;Water?
	JR Z NextCell		;Yes, skip it
	CP Fire			;Burning cell?
	JR NZ +4		;No, move on
	LD (HL),00		;Burning->Empty
	JR NextCell
	BIT 7,A			;Border chr?
	JR NZ NextCell		;Yes, skip it
	AND A			;Empty cell?
	JR NZ Strike		;No, move on, must be a tree at this point
				;Plant a tree?
	CALL RND8Bit		;Get random number, 0 to 255
@NewTree
	CP 00			;New tree?
	JR NC NextCell		;NC=No tree, move on
	LD A,"+"		;Tree chr
	LD (HL),A		;Plant a tree
	JR NextCell
@Strike				;Start a new fire?
	CALL RND8Bit		;Get random number, 0 to 255
@NewFire
	CP 00			;New fire??
	JR NC TestCells		;NC=No fire, move on
	LD A,Fire			;Fire chr
	LD (HL),A		;Start a new fire
	JR NextCell
;==============================
; Analyze the surrounding cells
;==============================
@TestCells
	AND A			;Clear CY just in case it's set
	SBC HL,BC		;Subtract row offet
	DEC HL			;NWcell
	CALL Burning		;Test for burning cell
	INC HL			;Ncell
	CALL Burning		;Test for burning cell
	INC HL			;NEcell
	CALL Burning		;Test for burning cell
	ADD HL,BC		;Ecell
	CALL Burning		;Test for burning cell
	ADD HL,BC		;SEcell
	CALL Burning		;Test for burning cell
	DEC HL			;Scell
	CALL Burning		;Test for burning cell
	DEC HL			;SWcell
	CALL Burning		;Test for burning cell
	AND A			;Clear CY just in case it's set
	SBC HL,BC		;Ecell
	CALL Burning		;Test for burning cell
@NextCell
	LD HL,(CenterCell+1)	;Get current cell position
	POP DE			;POP cell counter
	DEC DE 			;Decrease cell counter
	LD A,D 
	OR E 
	JR NZ Spread		;Loop until DE=0
	RET
;==============================
; Tree->Burning
; 
; BC unchange
;==============================
@Burning
	LD A,(HL)
	CP Fire			;Burning Neighbor?
	RET NZ			;No, return
@CenterCell			;Tree->Burning
	LD HL,0000		;Cell address
	LD (HL),Fire		;Set fire
@Wind
	LD DE,0001		;Wind speed
	DEC DE 			;Decrease counter
	LD A,D 
	OR E
	JR NZ Wind+3		;Loop until DE=0
	POP AF			;POP return address
	JR NextCell		;Jump, we don't need to test anymore neighbors
;================================
; 8-Bit Random Number Generator
;
; Reg...
;
;	BC not used
;	HL Restored on exit
;	DE Restored on exit
;	 A Random number, range 0 to 255
;================================
@Rnd8Bit
	PUSH HL			;PUSH HL
	PUSH DE			;PUSH DE
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+3),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	POP DE			;POP DE
	POP HL			;POP HL
	RET