;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;    Seeping Water Routines
;    Gregory C. Harder 2020
;================================

ORG 4084

@DFile=400C			;16396, Start of DFile
@Vars=4010			;16400, Start of vars
@Keyboard=02BB			;The 'KEYBOARD SCANNING' subroutine
@Frames=4034			;16436, Seed
@Dry=403C			;16444, Background character
@Wet=403D			;16445, Wet character
@Prob=403E			;16446, Wet probability
@Speed=403F			;16447, Seep speed
@WString=4040			;16448, W$ data

	JR Border		;16516
	JR CLS			;16518
	JR Analyze		;16520
	JR Seep			;16522
	JR Delay		;16524
;================================
; Clear screen replacement
;================================
@CLS
	LD A,(Dry)		;Get background chr
	LD (BKGChr+1),A		;Set background chr
	LD HL,(DFile)
	LD BC,0318		;Byte count
@CSLup
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +2		;Yes, skip it
@BKGChr
	LD (HL),00		;Clear this byte with Dry chr
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ CSLup		;Loop until BC=0
	RET
;==============================
; Print border
;==============================
@Border
	LD A,80			;Border character
	LD HL,(DFile)
	LD BC,1700		;24 Rows
@Line
	INC HL 
	LD (HL),A		;Border Chr
	LD DE,001F
	ADD HL,DE 
	LD (HL),A		;Border Chr
	INC HL 
	DJNZ LINE
	CALL Row
	LD HL,(DFile)
@Row
	INC HL
	LD B,20
@RowLup
	LD (HL),A		;Border Chr
	INC HL
	DJNZ RowLup
	RET
;================================
; Scroll Up
;================================
@Seep
	LD HL,(DFile)		;1st scroll screen up
	LD D,H
	LD E,L
	LD BC,0021 
	ADD HL,BC 
	LD BC,02F7 
	LDIR
	CALL Analyze		;Analyze row
	LD HL,(DFile)		;Get start of row 23 
	LD BC,02F8		;Offset to start of row 23
	ADD HL,BC
	EX DE,HL		;DE=start if row 23
	LD HL,(WString)		;Get W$ address
	LD BC,0020 		;Byte count, 32 chrs to copy
	LDIR			;Print W$
	CALL Delay
@WaitKey
	Call Keyboard		;Call ROM scan keybd.
	INC L			;Check for no key press, L=FF
	JR Z Seep		;If L=0 then Scan again.
	RET
;================================
; Delay loop
;================================
@Delay
	LD A,(Speed)		;Get seep speed
	LD D,A			;Set seep speed
	LD E,01
	DEC DE 			;Decrease counter
	LD A,D 
	OR E
	JR NZ Delay+6		;Loop until DE=0
	RET
;================================
; Analyze cell condition
;================================
@Analyze
	LD A,(Dry)		;Get dry chr
	LD (Dry1+1),A		;Set dry chr
	LD A,(Wet)		;Get wet chr
	LD (Wet1+2),A		;Set wet chr
	LD A,(Prob)		;Get wet probability
	LD (Soak+1),A		;Set wet probability
	LD HL,(Vars)		;Get start of W$
	INC HL			;Skip $ header
	INC HL
	INC HL
	LD (WString),HL		;Save WString address
	LD B,1E			;Byte count
@Wet1
	INC HL			;W$(X), chr to analyze
	LD C,00
	DEC HL			;W$(X-1), test left side
	LD A,(HL)		;Get chr there
	INC HL			;W$(X)
	CP C			;Left side=wet chr?
	CALL Z Drip		;Yes, make W$(X)?
	INC HL			;W$(X+1), test right side
	LD A,(HL)		;Get chr there
	DEC HL			;W$(X)
	CP C			;Right side=wet chr?
	CALL Z Drip		;Yes, make W$(X) wet?
	DJNZ Wet1
	RET
;================================
; Make W$(X) wet?
;================================
@Drip
	PUSH HL			;PUSH X$(X)	
	CALL RND8Bit		;Get random number, 0 to 255
	POP HL			;PUSH X$(X)	
@Soak
	CP 00			;Test against probability
	LD A,C			;A=wet chr
	JR C +2			;IF NC then make wet
@Dry1
	LD A,00			;W$(X)=dry chr
	LD (HL),A
	RET
;================================
; 8-Bit Random Number Generator
; Reg BC not used
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET